import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import { map } from 'rxjs/operators';

import { ToastrService } from 'ngx-toastr';
import { DataStateChangeEventArgs, DataResult } from '@syncfusion/ej2-angular-grids';

import { ClientService } from '../utils/client.service';
import { CodecService } from '../utils/codec.service';
import { ConfigSetting } from 'src/app/models/config-setting';
import { UiConfigHrow } from './ui-config-hrow';
import { ApiResponse } from 'src/app/models/api-response';

import { environment as env } from 'src/environments/environment';

@Injectable({
  providedIn: 'root'
})
export class ConfigService extends Subject<DataStateChangeEventArgs> {

  constructor(
    private client: ClientService,
    private codec: CodecService,
    private toaster: ToastrService
  ) {
    super();
  }

  public execute(state: DataStateChangeEventArgs): void {
    console.log('configService engaged for fetch data...');
    this.getAllConfig().subscribe(x => super.next(x));
  }

  protected getAllConfig(): Observable<DataStateChangeEventArgs> {
    const lacEndpoint = env.urls.api.su.suListAllCfg;
    let cfgCount = 0;
    return this.client.put(lacEndpoint, null)
      .pipe(map((apiResponse: ApiResponse) => this.codec.decode(apiResponse)))
      .pipe(map((apiResponse: ApiResponse) => {
        // console.log('Check Service Respnse: ' + JSON.stringify(apiResponse));
        if (apiResponse.s === env.const.STATUS_OK) {
          const cfgSettings: ConfigSetting[] = apiResponse.r;
          cfgCount = cfgSettings.length;
          this.client.setItem(env.const.CFG_SETTINGS, JSON.stringify(cfgSettings));
          this.client.persistChanges();
          return this.prepareView(cfgSettings);
        } else {
          this.toaster.error(apiResponse.c);
          return [];
        }
      }))
      .pipe(map((data: UiConfigHrow[]) => {
        console.log('converting to DataResult.... Check sections: ' + data.length);
        return ({
          result: data,
          count: cfgCount
        } as DataResult);
      }))
      .pipe((tblData: any) => tblData);
  }

  prepareView(settings: ConfigSetting[]): UiConfigHrow[] {
    const data: UiConfigHrow[] = [];
    for (const setting of settings) {
      const section: string = setting.s;
      const key: string = setting.n;
      const val: string = setting.v;

      let currSection: UiConfigHrow = null;
      let properties: { name: string; value: string; }[] = [];

      for (const row of data) {
        if (row.section === section) {
          currSection = row;
          properties = row.properties;
        }
      }

      if (currSection === null) {
        currSection = new UiConfigHrow();
        currSection.section = section;
        currSection.properties = properties;
        data.push(currSection);
      }
      properties.push({ name: key, value: val });
    }
    // console.log('Prepared tblData: ' + JSON.stringify(data));
    return data;
  }
}
