import { Component, OnInit, ViewChild, Inject } from '@angular/core';
import { Observable } from 'rxjs';

import { ToastrService } from 'ngx-toastr';
import { NgxSpinnerService } from 'ngx-spinner';
import { DataSourceChangedEventArgs, DataStateChangeEventArgs, Sorts } from '@syncfusion/ej2-grids';
import { SortSettingsModel, PageSettingsModel, EditSettingsModel, TreeGridComponent } from '@syncfusion/ej2-angular-treegrid';

import { CodecService } from 'src/app/modules/utils/codec.service';
import { ToolsService } from 'src/app/modules/utils/tools.service';
import { ClientService } from 'src/app/modules/utils/client.service';
import { ConfigService } from 'src/app/modules/super-user/config.service';

@Component({
  selector: 'dbw-list-all-cfg',
  templateUrl: './list-all-cfg.component.html',
  styleUrls: ['./list-all-cfg.component.css'],
  providers: [ConfigService]
})
export class ListAllCfgComponent implements OnInit {

  constructor(
    private codec: CodecService,
    private tools: ToolsService,
    private client: ClientService,
    private config: ConfigService,
    private spinner: NgxSpinnerService,
    private toastr: ToastrService
  ) {
    this.tblDataSource = this.config;
  }

  // model/ data binding...
  tblDataSource: Observable<DataStateChangeEventArgs>;

  // handle view...
  tblToolbarOptions: string[];
  tblEditSettings: EditSettingsModel;
  tblSortSettings: SortSettingsModel;
  tblPagingSettings: PageSettingsModel;
  tblCommands: Object[];

  // handle view and controller state...
  state: DataStateChangeEventArgs;

  ngOnInit() {
    this.tblSortSettings = {
      columns: [
        { field: 'section', direction: 'Ascending' },
        { field: 'name', direction: 'Ascending' }]
    };
    this.tblPagingSettings = { pageSize: 15 };
    this.tblToolbarOptions = ['Add', 'Edit', 'Delete', 'Update', 'Cancel', 'Search'];
    this.tblEditSettings = { allowEditing: true, allowAdding: true, allowDeleting: true, mode: 'Row' };
    this.tblCommands = [
      { type: 'Edit', buttonOption: { iconCss: ' e-icons e-edit', cssClass: 'e-flat' } },
      { type: 'Delete', buttonOption: { iconCss: 'e-icons e-delete', cssClass: 'e-flat' } },
      { type: 'Save', buttonOption: { iconCss: 'e-icons e-update', cssClass: 'e-flat' } },
      { type: 'Cancel', buttonOption: { iconCss: 'e-icons e-cancel-icon', cssClass: 'e-flat' } }];

    console.log('Going to invoke initial load of config...');
    this.config.execute(this.state);
  }

  onDataBound(args: any): void {
    console.log('data is bound... check event: ' + JSON.stringify(args));
  }

  refreshView(newState: DataStateChangeEventArgs) {
    console.log('UI DataState has changed...');
    console.log('Check dataStateChangedEvent: ' + JSON.stringify(newState));
    this.config.execute(newState);
  }

  handleCRUDEvent(crudEvent: DataSourceChangedEventArgs) {
    console.log('UI Data has changed...');
    console.log('crudEvent: ' + JSON.stringify(crudEvent));
    /* if (crudEvent.action === 'add' || crudEvent.action === 'edit') {
      if (this.saveSetting(crudEvent.data)) {
        crudEvent.endEdit();
      } else {
        crudEvent.cancelEdit();
      }
    }

    if (crudEvent.action === 'delete') {
      if (this.deleteSetting(crudEvent.data)) {
        crudEvent.endEdit();
      } else {
        crudEvent.cancelEdit();
      }
    } */
  }

  /* saveSetting(save: any): boolean {
    if (this.isBackendBusy) {
      this.toastr.error('Backend is busy; please try later!!');
      return;
    }
    let apiResponse: ApiResponse;
    const setting = new ConfigSetting(save.newData.s, save.newData.n, save.newData.v);
    console.log('Saving config setting: ' + JSON.stringify(setting));
    this.spinner.show();

    const suScsEndpoint = env.urls.api.su.suSaveCfgSetting;
    this.client.put(suScsEndpoint, setting)
      .subscribe(
        (data: ApiResponse) => apiResponse = this.codec.decode(data),
        error => () => { this.toastr.error('Save config setting failed. Cause: ' + error); this.spinner.hide(); },
        () => {
          console.log('Save Setting Service Response: ' + JSON.stringify(apiResponse));

          if (apiResponse.s === env.const.STATUS_OK) {
            this.cfgSettings.push(setting);
            this.client.setItem(env.const.CFG_SETTINGS, this.cfgSettings);
            this.client.persistChanges();
            this.spinner.hide();
            this.toastr.success('Saved Config Value!');
            return true;
          } else {
            this.toastr.error(apiResponse.c);
            return false;
          }
        }
      );
  }

  deleteSetting(toDelete: any): boolean {
    if (this.isBackendBusy) {
      this.toastr.error('Backend is busy; please try later!!');
      return;
    }
    let apiResponse: ApiResponse;
    const setting = new ConfigSetting(toDelete.s, toDelete.n, toDelete.v);
    console.log('Deleting config setting: ' + JSON.stringify(setting));
    this.spinner.show();
    const suDcEndpoint = env.urls.api.su.suDeleteCfgSetting;
    this.client.put(suDcEndpoint, setting)
      .subscribe(
        (data) => apiResponse = this.codec.decode(data),
        error => () => { this.toastr.error('Failed deleting config. Cause: ' + error); this.spinner.hide(); },
        () => {
          console.log('Delete Setting Service Response: ' + JSON.stringify(apiResponse));
          if (apiResponse.s === env.const.STATUS_OK) {
            this.cfgSettings = this.tools.removeFromArray(this.cfgSettings, setting);
            this.client.setItem(env.const.CFG_SETTINGS, this.cfgSettings);
            this.client.persistChanges();
            this.spinner.hide();
            this.toastr.success('Deleted Config Value!');
            return true;
          } else {
            this.toastr.error(apiResponse.c);
            return false;
          }
        }
      );
  } */
}
