import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { RouterModule } from '@angular/router';
import { FormsModule } from '@angular/forms';
import { HttpClient } from '@angular/common/http';

import { TranslateLoader, TranslateModule, TranslateService } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { NgxSpinnerModule, NgxSpinnerService } from 'ngx-spinner';
import { ButtonModule } from '@syncfusion/ej2-angular-buttons';
import { DropDownListAllModule } from '@syncfusion/ej2-angular-dropdowns';
import { GridModule } from '@syncfusion/ej2-angular-grids';
import {
  TreeGridModule, PageService, SortService, FilterService, ToolbarService, EditService, CommandColumnService
} from '@syncfusion/ej2-angular-treegrid';

import { AdminHomeComponent } from '../../segments/super-user/admin-home/admin-home.component';
import { ListAllCfgComponent } from '../../segments/super-user/list-all-cfg/list-all-cfg.component';
import { AssignAdminComponent } from '../../segments/super-user/assign-admin/assign-admin.component';
import { ConfigService } from './config.service';


@NgModule({
  declarations: [
    AdminHomeComponent,
    ListAllCfgComponent,
    AssignAdminComponent
  ],

  imports: [
    CommonModule,
    RouterModule,
    FormsModule,
    NgxSpinnerModule,
    GridModule,
    TreeGridModule,
    ButtonModule,
    DropDownListAllModule,
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    })
  ],

  exports: [
    AdminHomeComponent,
    ListAllCfgComponent,
    AssignAdminComponent
  ],

  providers: [
    NgxSpinnerService,
    PageService,
    SortService,
    FilterService,
    ToolbarService,
    EditService,
    CommandColumnService,

    ConfigService
  ]
})
export class SuperUserModule {
  constructor(
    private translate: TranslateService
  ) {
    // Gets Default language from browser if available, otherwise set English ad default
    this.translate.addLangs(['en', 'ms']);
    this.translate.setDefaultLang('en');
    const browserLang = this.translate.getBrowserLang();
    this.translate.use(browserLang.match(/en|en-US|ms|ms-my/) ? browserLang : 'en');
  }
}

export function HttpLoaderFactory(http: HttpClient) {
  return new TranslateHttpLoader(http);
}
